/*  Copyright (c) 2016 Michael Hansen

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE. */

#ifndef _ST_CONFIG_H
#define _ST_CONFIG_H

#define ST_MAJOR_VERSION 1
#define ST_MINOR_VERSION 0
#define ST_VERSION ((ST_MAJOR_VERSION * 100) + ST_MINOR_VERSION)
#define ST_VERSION_STR "1.0"

/* #undef ST_HAVE_NOEXCEPT */
#define ST_HAVE_OVERRIDE
#define ST_HAVE_NULLPTR
#define ST_HAVE_STATIC_ASSERT
#define ST_HAVE_RVALUE_MOVE
/* #undef ST_HAVE_CHAR_TYPES */
/* #undef ST_HAVE_USER_LITERALS */
#define ST_HAVE_INT64
#define ST_HAVE_DELETED_FUNC
#define ST_HAVE_VARARG_TEMPLATE
#define ST_HAVE_TYPE_TRAITS
/* #undef ST_HAVE_CONSTEXPR */
/* #undef ST_INT64_IS_LONG */
#define ST_BUILD_SHARED 0

#ifdef ST_HAVE_NOEXCEPT
#   define ST_NOEXCEPT noexcept
#else
#   define ST_NOEXCEPT throw()
#endif

#ifdef ST_HAVE_OVERRIDE
#   define ST_OVERRIDE override
#else
#   define ST_OVERRIDE
#endif

#ifdef ST_HAVE_NULLPTR
#   define ST_NULLPTR nullptr
#else
#   define ST_NULLPTR 0
#endif

#ifdef ST_HAVE_STATIC_ASSERT
#   define ST_STATIC_ASSERT(x, msg) static_assert((x), msg);
#else
#   define ST_STATIC_ASSERT(x, msg)
#endif

#ifndef ST_HAVE_CHAR_TYPES
    typedef unsigned short  char16_t;
    typedef unsigned int    char32_t;
#endif

#ifdef ST_HAVE_DELETED_FUNC
#   define ST_DISABLE_COPY(name) \
        name(const name &) = delete; \
        void operator=(const name &) = delete;
#else
#   define ST_DISABLE_COPY(name) \
    private: \
        name(const name &) { } \
        void operator=(const name &) { }
#endif

#ifdef ST_HAVE_CONSTEXPR
#   define ST_STRONG_ENUM(name) enum class name
#   define ST_ENUM_CONSTANT(type, name) constexpr type name = type::name
#else
#   define ST_STRONG_ENUM(name) enum name
#   define ST_ENUM_CONSTANT(type, name)
#endif

#define ST_WCHAR_BYTES          (2)
#define ST_SIZET_BYTES          (4)
#define ST_SHORT_STRING_LEN     (16)
#define ST_STACK_STRING_LEN     (256)

// MSVC doesn't provide ssize_t
#ifdef _MSC_VER
#   ifdef _WIN64
        typedef __int64 ST_ssize_t;
#   else
        typedef int ST_ssize_t;
#   endif
#else
#   include <sys/types.h>
    typedef ssize_t ST_ssize_t;
#endif

#if ST_BUILD_SHARED && defined(WIN32)
#   ifdef ST_BUILD_EXPORTS
#       define ST_EXPORT __declspec(dllexport)
#   else
#       define ST_EXPORT __declspec(dllimport)
#   endif
#else
#   define ST_EXPORT
#endif

#endif // _ST_CONFIG_H
